import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Date;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.LongField;
import org.apache.lucene.document.StringField;
import org.apache.lucene.document.TextField;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.index.IndexWriterConfig.OpenMode;
import org.apache.lucene.index.Term;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class BasicIndex {
	IndexWriter writer_ = null;


	/**
	 * construct a indexer with index path
	 */
	public BasicIndex(String indexPath) {
		try {
			Directory indexDir = FSDirectory.open(new File(indexPath)); // create a index on disk
			Analyzer analyzer = new StandardAnalyzer(Version.LUCENE_45); // use standard analyzer
			IndexWriterConfig iwc = new IndexWriterConfig(Version.LUCENE_45, analyzer); // config the index writer
			iwc.setOpenMode(OpenMode.CREATE); // set open mode to create a new index

			writer_ = new IndexWriter(indexDir, iwc); // create a writer

		} catch (IOException e) {
			System.out.println(e.getMessage());
			System.exit(1);
		}
	}


	/**
	 * clsee the index writer
	 */
	public void close() throws Exception {
		try {
			writer_.close();

		} catch (IOException e) {
			System.out.println(e.getMessage());
			writer_.close();
			System.exit(1);

		} finally {
			writer_.close();
		}
	}


	/**
	 * create index for a given regular file
	 */
	public void indexDoc(File file) throws Exception {
		FileInputStream fis = null;

		try { 
			fis = new FileInputStream(file);
			Document doc = new Document();

			// add a field to the doc
			Field nameField = new StringField("name", file.getName(), Field.Store.YES);
			doc.add(nameField);

			// add a field to the doc
			Field contentField = new TextField("content", new BufferedReader(new InputStreamReader(fis, "UTF-8")));
			doc.add(contentField);

			if (writer_.getConfig().getOpenMode() == OpenMode.CREATE) {
				writer_.addDocument(doc); // create index for the given file

			} else {
				Term term = new Term("name", file.getPath());
				writer_.updateDocument(term, doc);
			}

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
			fis.close();
			System.exit(1);

		} finally {
			fis.close();
		}
	}


	/**
	 * travers a path and create index for every regular file under the path
	 */
	public void indexDocs(String docsPath) throws Exception {
		File docsDir = new File(docsPath);
		if(docsDir.isFile()) { // regular file, create index
			indexDoc(docsDir);

		} else if(docsDir.isDirectory()) { // directory, traves every file in the directory
			File[] files = docsDir.listFiles();

			for(int i = 0; i < files.length; i++) {
				indexDocs(files[i].getPath());
			}
		}
	}



	public static void main(String[] args) throws Exception {
		String docsPath = args[0];
		String indexPath = args[1];

		try {
			BasicIndex bi = new BasicIndex(indexPath);
			bi.indexDocs(docsPath);
			bi.close();

		} catch (IOException e) {
			System.out.println(e.getMessage());
		}
	}
}
