import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Date;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.similarities.Similarity;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class BasicSearch {
	IndexReader reader_ = null;
	IndexSearcher searcher_ = null;
	Analyzer analyzer_ = null;


	/**
	 * constructor
	 */
	public BasicSearch(String indexPath) throws Exception {
		reader_ = DirectoryReader.open(FSDirectory.open(new File(indexPath))); // create a index reader
		searcher_ = new IndexSearcher(reader_); // create index searcher
		analyzer_ = new StandardAnalyzer(Version.LUCENE_45); // create analyzer
	}


	/**
	 * close the index reader
	 */
	public void close() throws Exception {
		reader_.close();
	}


	public void useWeirdSimilarity() {
		Similarity similarity = new WeirdSimilarity();
		searcher_.setSimilarity(similarity);
	}


	/**
	 * do term query
	 */
	public void doTermQuery(String field, String word, int top) throws Exception {
		Term term = new Term(field, word);
		TermQuery tq = new TermQuery(term);

		TopDocs topDocs = null;

		if(top < 0) { // get all the results
			topDocs = searcher_.search(tq, 1);
			int totalHits = topDocs.totalHits;
			topDocs = searcher_.search(tq, totalHits);
		
		} else { // get the top rank results
			topDocs = searcher_.search(tq, top);
		}

		ScoreDoc[] scoreDocs = topDocs.scoreDocs;
		for(int i = 0; i < scoreDocs.length; i++) {
			Document doc = searcher_.doc(scoreDocs[i].doc);
			System.out.println(doc.get("name"));
		}
	}


	/**
	 * test program
	 */
	public static void main(String[] args) throws Exception {
		String indexPath = args[0];
		String mode = args[1];
		String field = args[2];
		String query = args[3];
		int top = Integer.valueOf(args[4]);

		BasicSearch bs = new BasicSearch(indexPath);

        if(mode.equals("weird")) {
		    bs.useWeirdSimilarity();

        }

		bs.doTermQuery(field, query, top);
		bs.close();
	}
}
