import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Date;

import org.apache.lucene.search.similarities.*;
import org.apache.lucene.index.FieldInvertState;
import org.apache.lucene.util.BytesRef;
import org.apache.lucene.util.SmallFloat;

public class WeirdSimilarity extends TFIDFSimilarity {
	/** Cache of decoded bytes. */
	private static final float[] NORM_TABLE = new float[256];

	static {
		for (int i = 0; i < 256; i++) {
			NORM_TABLE[i] = SmallFloat.byte315ToFloat((byte)i);
		}
	}


	public WeirdSimilarity() {}
	

	/** Implemented as <code>overlap / maxOverlap</code>. */
	@Override
	public float coord(int overlap, int maxOverlap) {
		return overlap / (float)maxOverlap;
	}


	/** Implemented as <code>1/sqrt(sumOfSquaredWeights)</code>. */
	@Override
	public float queryNorm(float sumOfSquaredWeights) {
		return (float)(1.0 / Math.sqrt(sumOfSquaredWeights));
	}
	

	@Override
	public final long encodeNormValue(float f) {
		return SmallFloat.floatToByte315(f);
	}


	@Override
	public final float decodeNormValue(long norm) {
		return NORM_TABLE[(int) (norm & 0xFF)];	// & 0xFF maps negative bytes to positive above 127
	}


	@Override
	public float lengthNorm(FieldInvertState state) {
		final int numTerms;

		if (discountOverlaps) {
			numTerms = state.getLength() - state.getNumOverlap();

		} else {
			numTerms = state.getLength();
		}

		return state.getBoost() * ((float) (1.0 / Math.sqrt(numTerms)));
	}


	@Override
	public float tf(float freq) {
		//return (float)Math.sqrt(freq);
		return (float)(1.0 / (Math.sqrt(freq) + 1.0));
	}

		
	@Override
	public float sloppyFreq(int distance) {
		return 1.0f / (distance + 1);
	}
	

	@Override
	public float scorePayload(int doc, int start, int end, BytesRef payload) {
		return 1;
	}


	@Override
	public float idf(long docFreq, long numDocs) {
		return (float)(Math.log(numDocs/(double)(docFreq+1)) + 1.0);
	}
		

	protected boolean discountOverlaps = true;


	public void setDiscountOverlaps(boolean v) {
		discountOverlaps = v;
	}


	public boolean getDiscountOverlaps() {
		return discountOverlaps;
	}


	@Override
	public String toString() {
		return "StupidSimilarity";
	}
}
